'use strict';

const index = require('./index-00f0fc21.js');
const utils = require('./utils-a086ed6e.js');
const getQueryArg = require('./get-query-arg-53bf21e2.js');
const index$1 = require('./index-8604ebe6.js');
const defaultI18n = require('./default-i18n-2d921004.js');
const google = require('./google-4d37e025.js');
const currency = require('./currency-ba038e2f.js');
const addQueryArgs = require('./add-query-args-17c551b6.js');

/**
 * Internal dependencies
 */
/**
 * Removes arguments from the query string of the url
 *
 * @param {string}    url  URL.
 * @param {...string} args Query Args.
 *
 * @example
 * ```js
 * const newUrl = removeQueryArgs( 'https://wordpress.org?foo=bar&bar=baz&baz=foobar', 'foo', 'bar' ); // https://wordpress.org?baz=foobar
 * ```
 *
 * @return {string} Updated URL.
 */

function removeQueryArgs(url) {
  const queryStringIndex = url.indexOf('?');

  if (queryStringIndex === -1) {
    return url;
  }

  const query = addQueryArgs.getQueryArgs(url);
  const baseURL = url.substr(0, queryStringIndex);

  for (var _len = arguments.length, args = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    args[_key - 1] = arguments[_key];
  }

  args.forEach(arg => delete query[arg]);
  const queryString = addQueryArgs.buildQueryString(query);
  return queryString ? baseURL + '?' + queryString : baseURL;
}

const safeRead = (storage, key) => {
  try {
    return JSON.parse(storage.getItem(key));
  }
  catch {
    return null;
  }
};
const debounce = fn => {
  let called = false;
  return () => {
    if (called) {
      return;
    }
    called = true;
    setTimeout(() => {
      fn();
      called = false;
    }, 0);
  };
};
const createStorageStore = (storage, key, defaultValues, syncAcrossTabs = false) => {
  var _a;
  const store = index.createStore((_a = safeRead(storage, key)) !== null && _a !== void 0 ? _a : defaultValues, (newValue, oldValue) => {
    return JSON.stringify(newValue) !== JSON.stringify(oldValue);
  });
  // Need to sync later or we would get ourselves in an infinite loop.
  const write = debounce(() => storage.setItem(key, JSON.stringify(store.state)));
  write();
  if (syncAcrossTabs) {
    window.addEventListener('storage', () => {
      const currentState = safeRead(storage, key);
      if (currentState === null) {
        return;
      }
      for (const key in currentState) {
        store.set(key, currentState[key]);
      }
    });
  }
  store.use({ set: write, reset: write });
  return store;
};
const createLocalStore = (key, defaultValues, syncAcrossTabs = false) => createStorageStore(localStorage, key, defaultValues, syncAcrossTabs);

const { checkout: checkout$1 } = utils.getSerializedState();
const notPersistCart = (checkout$1 === null || checkout$1 === void 0 ? void 0 : checkout$1.persist) !== 'browser' || !!getQueryArg.getQueryArg(window.location.href, 'no_cart');
const store$1 = notPersistCart
  ? index.createStore({
    live: {},
    test: {},
  })
  : createLocalStore('surecart-local-storage', {
    live: {},
    test: {},
  }, true);
const { state: state$2, onChange: onChange$1, on: on$2, set: set$1, get: get$1, dispose: dispose$1 } = store$1;
window.scStore = store$1;

const { checkout } = utils.getSerializedState();
const { state: state$1, onChange, on: on$1, set, get, dispose } = index.createStore({
  formId: null,
  groupId: null,
  mode: 'live',
  locks: [],
  product: null,
  checkout: null,
  currencyCode: 'usd',
  abandonedCheckoutEnabled: true,
  initialLineItems: [],
  isCheckoutPage: false,
  validateStock: false,
  persist: 'browser',
  ...checkout,
}, (newValue, oldValue) => {
  return JSON.stringify(newValue) !== JSON.stringify(oldValue);
});

/**
 * When the checkout changes, update the
 * checkout in localstorage.
 */
onChange('checkout', val => setCheckout(val, state$1.formId));
/**
 * When checkout is get, get the checkout from the checkouts state.
 */
on$1('get', prop => {
  if (prop === 'checkout') {
    const checkout = getCheckout(state$1.formId, state$1.mode);
    if (checkout === null || checkout === void 0 ? void 0 : checkout.id) {
      state$1.checkout = checkout;
    }
  }
});

const store = index.createStore(() => ({
  cart: {
    open: false,
  },
}), (newValue, oldValue) => {
  return JSON.stringify(newValue) !== JSON.stringify(oldValue);
});
const { state } = store;
const toggleCart = (open = null) => store.set('cart', { ...state.cart, ...{ open: open !== null ? open : !state.cart.open } });
const { on } = store;
on('set', (key, newState) => {
  if (key !== 'cart')
    return; // we only care about cart.
  if (newState === null || newState === void 0 ? void 0 : newState.open) {
    index$1.speak(defaultI18n.__('Cart Opened', 'surecart'), 'assertive');
  }
  else {
    index$1.speak(defaultI18n.__('Cart Closed', 'surecart'), 'assertive');
  }
});

// Listen to all stored checkouts and handle line item changes.
on$2('set', (_, value, oldValue) => Object.keys(value || {}).forEach(key => handleCheckoutLineItemChange(value[key], oldValue === null || oldValue === void 0 ? void 0 : oldValue[key])));
/**
 * Add to cart/remove from cart, cart updated events.
 */
const handleCheckoutLineItemChange = (checkout, oldCheckout) => {
  var _a, _b;
  // get new and old line items.
  const newLineItems = ((_a = checkout === null || checkout === void 0 ? void 0 : checkout.line_items) === null || _a === void 0 ? void 0 : _a.data) || [];
  const oldLineItems = ((_b = oldCheckout === null || oldCheckout === void 0 ? void 0 : oldCheckout.line_items) === null || _b === void 0 ? void 0 : _b.data) || [];
  // check for added items or quantity changed
  newLineItems.forEach(newItem => {
    const oldItem = oldLineItems.find(item => item.id === newItem.id);
    // an item was added, or the quantity changed.
    if (!oldItem || (oldItem === null || oldItem === void 0 ? void 0 : oldItem.quantity) < (newItem === null || newItem === void 0 ? void 0 : newItem.quantity)) {
      const event = new CustomEvent('scAddedToCart', {
        detail: {
          ...newItem,
          quantity: newItem.quantity - ((oldItem === null || oldItem === void 0 ? void 0 : oldItem.quantity) || 0),
        },
        bubbles: true,
      });
      document.dispatchEvent(event);
    }
  });
  // check for removed items or quantity changed
  oldLineItems.forEach(oldItem => {
    const newItem = newLineItems.find(item => item.id === oldItem.id);
    // an item was removed, or the quantity changed.
    if (!newItem || (oldItem === null || oldItem === void 0 ? void 0 : oldItem.quantity) > (newItem === null || newItem === void 0 ? void 0 : newItem.quantity)) {
      const event = new CustomEvent('scRemovedFromCart', {
        detail: {
          ...oldItem,
          quantity: oldItem.quantity - ((newItem === null || newItem === void 0 ? void 0 : newItem.quantity) || 0),
        },
        bubbles: true,
      });
      document.dispatchEvent(event);
    }
  });
  // check if line items have changed.
  if (!(state$1 === null || state$1 === void 0 ? void 0 : state$1.isCheckoutPage) && JSON.stringify(newLineItems) !== JSON.stringify(oldLineItems)) {
    // emit an event here with the checkout state updates.
    const event = new CustomEvent('scCartUpdated', {
      detail: {
        currentCart: checkout,
        previousCart: oldCheckout,
      },
      bubbles: true,
    });
    document.dispatchEvent(event);
  }
};
/**
 * Handle view cart event.
 */
on('set', (key, newValue, oldValue) => {
  if (key !== 'cart')
    return; // we only care about cart.
  if ((newValue === null || newValue === void 0 ? void 0 : newValue.open) === (oldValue === null || oldValue === void 0 ? void 0 : oldValue.open))
    return; // we only care about open state changes.
  if (newValue === null || newValue === void 0 ? void 0 : newValue.open) {
    const event = new CustomEvent('scViewedCart', {
      detail: state$1.checkout,
      bubbles: true,
    });
    document.dispatchEvent(event);
  }
});

/**
 * Checkout initiated event.
 */
on$1('set', (key, checkout, oldCheckout) => {
  if (key !== 'checkout')
    return; // we only care about checkout
  if (oldCheckout === null || oldCheckout === void 0 ? void 0 : oldCheckout.id)
    return; // we only care about new checkouts.
  if (!(checkout === null || checkout === void 0 ? void 0 : checkout.id))
    return; // we don't have a saved checkout.
  if (!state$1.isCheckoutPage)
    return; // we don't want to fire this if we are not on the checkout page.
  const event = new CustomEvent('scCheckoutInitiated', {
    detail: checkout,
    bubbles: true,
  });
  document.dispatchEvent(event);
});
/**
 * Purchase complete, trial start event.
 */
on$1('set', (key, checkout, oldCheckout) => {
  var _a, _b;
  if (key !== 'checkout')
    return; // we only care about checkout
  if (!(checkout === null || checkout === void 0 ? void 0 : checkout.status) || (oldCheckout === null || oldCheckout === void 0 ? void 0 : oldCheckout.status) === (checkout === null || checkout === void 0 ? void 0 : checkout.status))
    return; // we only care about status changes.
  if (!['paid', 'processing'].includes(checkout.status))
    return; // only if it's paid or processing.
  // order paid is deprecated.
  const deprecated = new CustomEvent('scOrderPaid', { detail: checkout, bubbles: true });
  document.dispatchEvent(deprecated);
  // emit the new event.
  const event = new CustomEvent('scCheckoutCompleted', { detail: checkout, bubbles: true });
  document.dispatchEvent(event);
  // get trial line items and emit trial event if there are any.
  const trialLineItems = (((_a = checkout === null || checkout === void 0 ? void 0 : checkout.line_items) === null || _a === void 0 ? void 0 : _a.data) || []).filter(item => { var _a; return ((_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.trial_duration_days) > 0; });
  if (trialLineItems.length > 0) {
    const event = new CustomEvent('scTrialStarted', { detail: trialLineItems, bubbles: true });
    document.dispatchEvent(event);
  }
  // get subscription line items and emit subscription event if there are any.
  const subscriptionLineItems = (((_b = checkout === null || checkout === void 0 ? void 0 : checkout.line_items) === null || _b === void 0 ? void 0 : _b.data) || []).filter(item => { var _a; return ((_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.recurring_interval_count) > 0; });
  if (subscriptionLineItems.length > 0) {
    const event = new CustomEvent('scSubscriptionStarted', { detail: subscriptionLineItems, bubbles: true });
    document.dispatchEvent(event);
  }
});
/**
 * Shipping info added event.
 */
on$1('set', (key, checkout, oldCheckout) => {
  if (key !== 'checkout')
    return; // we only care about checkout
  if (!state$1.isCheckoutPage)
    return; // we don't want to fire this if we are not on the checkout page.
  if (!(checkout === null || checkout === void 0 ? void 0 : checkout.selected_shipping_choice))
    return; // we only care about shipping info.
  if ((oldCheckout === null || oldCheckout === void 0 ? void 0 : oldCheckout.selected_shipping_choice) === (checkout === null || checkout === void 0 ? void 0 : checkout.selected_shipping_choice))
    return; // we only care about new shipping info.
  const event = new CustomEvent('scShippingInfoAdded', {
    detail: checkout,
    bubbles: true,
  });
  document.dispatchEvent(event);
});
/**
 * Checkout updated event.
 */
on$1('set', (key, checkout, oldCheckout) => {
  if (key !== 'checkout')
    return; // we only care about checkout
  if (!state$1.isCheckoutPage)
    return; // we don't want to fire this if we are not on the checkout page.
  if (!(oldCheckout === null || oldCheckout === void 0 ? void 0 : oldCheckout.id))
    return; // we don't have a saved checkout.
  if (JSON.stringify(checkout) === JSON.stringify(oldCheckout))
    return; // we only care about changes.
  const event = new CustomEvent('scCheckoutUpdated', {
    detail: {
      currentCheckout: checkout,
      previousCheckout: oldCheckout,
    },
    bubbles: true,
  });
  document.dispatchEvent(event);
});

/**
 * Handle add to cart event.
 */
window.addEventListener('scAddedToCart', (e) => {
  var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
  const item = e.detail;
  // sanity check.
  if (!((_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product))
    return;
  google.trackEvent('add_to_cart', {
    currency: (_b = item.price) === null || _b === void 0 ? void 0 : _b.currency,
    value: currency.maybeConvertAmount(((_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.amount) || 0, ((_d = item.price) === null || _d === void 0 ? void 0 : _d.currency) || 'USD'),
    items: [
      {
        item_id: (_f = (_e = item.price) === null || _e === void 0 ? void 0 : _e.product) === null || _f === void 0 ? void 0 : _f.id,
        item_name: (_h = (_g = item.price) === null || _g === void 0 ? void 0 : _g.product) === null || _h === void 0 ? void 0 : _h.name,
        item_variant: (item.variant_options || []).join(' / '),
        price: currency.maybeConvertAmount(((_j = item === null || item === void 0 ? void 0 : item.price) === null || _j === void 0 ? void 0 : _j.amount) || 0, ((_k = item.price) === null || _k === void 0 ? void 0 : _k.currency) || 'USD'),
        currency: (_l = item.price) === null || _l === void 0 ? void 0 : _l.currency,
        quantity: item.quantity,
        discount: (item === null || item === void 0 ? void 0 : item.discount_amount) ? currency.maybeConvertAmount((item === null || item === void 0 ? void 0 : item.discount_amount) || 0, ((_m = item.price) === null || _m === void 0 ? void 0 : _m.currency) || 'USD') : 0,
      },
    ],
  });
});
/**
 * Handle remove from cart event.
 */
window.addEventListener('scRemovedFromCart', (e) => {
  var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
  const item = e.detail;
  // sanity check.
  if (!((_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product))
    return;
  google.trackEvent('remove_from_cart', {
    currency: (_b = item.price) === null || _b === void 0 ? void 0 : _b.currency,
    value: currency.maybeConvertAmount(((_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.amount) || 0, ((_d = item.price) === null || _d === void 0 ? void 0 : _d.currency) || 'USD'),
    items: [
      {
        item_id: (_f = (_e = item.price) === null || _e === void 0 ? void 0 : _e.product) === null || _f === void 0 ? void 0 : _f.id,
        item_name: (_h = (_g = item.price) === null || _g === void 0 ? void 0 : _g.product) === null || _h === void 0 ? void 0 : _h.name,
        item_variant: (item.variant_options || []).join(' / '),
        price: currency.maybeConvertAmount(((_j = item === null || item === void 0 ? void 0 : item.price) === null || _j === void 0 ? void 0 : _j.amount) || 0, ((_k = item.price) === null || _k === void 0 ? void 0 : _k.currency) || 'USD'),
        currency: (_l = item.price) === null || _l === void 0 ? void 0 : _l.currency,
        quantity: item.quantity,
        discount: (item === null || item === void 0 ? void 0 : item.discount_amount) ? currency.maybeConvertAmount((item === null || item === void 0 ? void 0 : item.discount_amount) || 0, ((_m = item.price) === null || _m === void 0 ? void 0 : _m.currency) || 'USD') : 0,
      },
    ],
  });
});
/**
 * Handle view cart event.
 */
window.addEventListener('scViewedCart', (e) => {
  var _a;
  const checkout = e.detail;
  google.trackEvent('view_cart', {
    currency: checkout.currency,
    value: currency.maybeConvertAmount(checkout.total_amount, checkout.currency),
    items: (((_a = checkout.line_items) === null || _a === void 0 ? void 0 : _a.data) || []).map(item => {
      var _a, _b, _c, _d, _e, _f, _g, _h, _j;
      return ({
        item_id: (_b = (_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.id,
        item_name: (_d = (_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.name,
        currency: (_e = item.price) === null || _e === void 0 ? void 0 : _e.currency,
        discount: item.discount_amount ? currency.maybeConvertAmount(item.discount_amount, (_f = item.price) === null || _f === void 0 ? void 0 : _f.currency) : 0,
        price: currency.maybeConvertAmount((_g = item === null || item === void 0 ? void 0 : item.price) === null || _g === void 0 ? void 0 : _g.amount, (_h = item.price) === null || _h === void 0 ? void 0 : _h.currency),
        quantity: item.quantity,
        ...(((_j = item === null || item === void 0 ? void 0 : item.variant_options) === null || _j === void 0 ? void 0 : _j.length) ? { item_variant: (item.variant_options || []).join(' / ') } : {}),
      });
    }),
  });
});
/**
 * Handle checkout initiated event.
 */
window.addEventListener('scCheckoutInitiated', (e) => {
  var _a;
  const checkout = e.detail;
  google.trackEvent('begin_checkout', {
    currency: checkout.currency,
    value: currency.maybeConvertAmount(checkout.total_amount, checkout.currency),
    items: (((_a = checkout.line_items) === null || _a === void 0 ? void 0 : _a.data) || []).map(item => {
      var _a, _b, _c, _d, _e, _f, _g, _h, _j;
      return ({
        item_id: (_b = (_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.id,
        item_name: (_d = (_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.name,
        currency: (_e = item.price) === null || _e === void 0 ? void 0 : _e.currency,
        discount: item.discount_amount ? currency.maybeConvertAmount(item.discount_amount, (_f = item.price) === null || _f === void 0 ? void 0 : _f.currency) : 0,
        price: currency.maybeConvertAmount((_g = item === null || item === void 0 ? void 0 : item.price) === null || _g === void 0 ? void 0 : _g.amount, (_h = item.price) === null || _h === void 0 ? void 0 : _h.currency),
        quantity: item.quantity,
        ...(((_j = item === null || item === void 0 ? void 0 : item.variant_options) === null || _j === void 0 ? void 0 : _j.length) ? { item_variant: (item.variant_options || []).join(' / ') } : {}),
      });
    }),
  });
});
/**
 * Handle purchase complete event.
 */
window.addEventListener('scCheckoutCompleted', (e) => {
  var _a;
  const checkout = e.detail;
  google.trackEvent('purchase', {
    transaction_id: checkout === null || checkout === void 0 ? void 0 : checkout.id,
    value: currency.maybeConvertAmount(checkout === null || checkout === void 0 ? void 0 : checkout.total_amount, (checkout === null || checkout === void 0 ? void 0 : checkout.currency) || 'USD'),
    currency: (checkout.currency || '').toUpperCase(),
    items: (((_a = checkout === null || checkout === void 0 ? void 0 : checkout.line_items) === null || _a === void 0 ? void 0 : _a.data) || []).map(item => {
      var _a, _b, _c, _d, _e, _f, _g, _h;
      return ({
        item_id: (_b = (_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.id,
        currency: (checkout.currency || '').toUpperCase(),
        item_name: ((_d = (_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.name) || '',
        discount: (item === null || item === void 0 ? void 0 : item.discount_amount) ? currency.maybeConvertAmount((item === null || item === void 0 ? void 0 : item.discount_amount) || 0, ((_e = item === null || item === void 0 ? void 0 : item.price) === null || _e === void 0 ? void 0 : _e.currency) || 'USD') : 0,
        price: currency.maybeConvertAmount(((_f = item === null || item === void 0 ? void 0 : item.price) === null || _f === void 0 ? void 0 : _f.amount) || 0, ((_g = item === null || item === void 0 ? void 0 : item.price) === null || _g === void 0 ? void 0 : _g.currency) || 'USD'),
        quantity: (item === null || item === void 0 ? void 0 : item.quantity) || 1,
        ...(((_h = item === null || item === void 0 ? void 0 : item.variant_options) === null || _h === void 0 ? void 0 : _h.length) ? { item_variant: (item.variant_options || []).join(' / ') } : {}),
      });
    }),
  });
});
/**
 * Handle payment info added event.
 */
window.addEventListener('scPaymentInfoAdded', (e) => {
  var _a;
  const checkout = e.detail;
  google.trackEvent('add_payment_info', {
    currency: checkout.currency,
    value: currency.maybeConvertAmount(checkout.total_amount, checkout.currency),
    items: (((_a = checkout.line_items) === null || _a === void 0 ? void 0 : _a.data) || []).map(item => {
      var _a, _b, _c, _d, _e, _f, _g, _h, _j;
      return ({
        item_id: (_b = (_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.id,
        item_name: ((_d = (_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.name) || '',
        currency: (_e = item.price) === null || _e === void 0 ? void 0 : _e.currency,
        discount: item.discount_amount ? currency.maybeConvertAmount(item.discount_amount, (_f = item.price) === null || _f === void 0 ? void 0 : _f.currency) : 0,
        price: currency.maybeConvertAmount((_g = item === null || item === void 0 ? void 0 : item.price) === null || _g === void 0 ? void 0 : _g.amount, (_h = item.price) === null || _h === void 0 ? void 0 : _h.currency),
        quantity: item.quantity,
        ...(((_j = item === null || item === void 0 ? void 0 : item.variant_options) === null || _j === void 0 ? void 0 : _j.length) ? { item_variant: (item.variant_options || []).join(' / ') } : {}),
      });
    }),
  });
});
/**
 * Handle shipping info added event.
 */
window.addEventListener('scShippingInfoAdded', (e) => {
  var _a, _b, _c, _d, _e, _f, _g, _h;
  const checkout = e.detail;
  const selectedShippingChoice = (_b = (_a = checkout === null || checkout === void 0 ? void 0 : checkout.shipping_choices) === null || _a === void 0 ? void 0 : _a.data) === null || _b === void 0 ? void 0 : _b.find(method => method.id === (checkout === null || checkout === void 0 ? void 0 : checkout.selected_shipping_choice));
  const selectedShippingTier = ((_c = selectedShippingChoice === null || selectedShippingChoice === void 0 ? void 0 : selectedShippingChoice.shipping_method) === null || _c === void 0 ? void 0 : _c.name) || '';
  google.trackEvent('add_shipping_info', {
    currency: checkout.currency,
    value: currency.maybeConvertAmount(checkout.total_amount, checkout.currency),
    ...(((_e = (_d = checkout === null || checkout === void 0 ? void 0 : checkout.discount) === null || _d === void 0 ? void 0 : _d.promotion) === null || _e === void 0 ? void 0 : _e.code) ? { coupon: (_g = (_f = checkout === null || checkout === void 0 ? void 0 : checkout.discount) === null || _f === void 0 ? void 0 : _f.promotion) === null || _g === void 0 ? void 0 : _g.code } : {}),
    ...(selectedShippingTier ? { shipping_tier: selectedShippingTier } : ''),
    items: (((_h = checkout.line_items) === null || _h === void 0 ? void 0 : _h.data) || []).map(item => {
      var _a, _b, _c, _d, _e, _f, _g, _h, _j;
      return ({
        item_id: (_b = (_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.id,
        item_name: ((_d = (_c = item === null || item === void 0 ? void 0 : item.price) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.name) || '',
        currency: (_e = item.price) === null || _e === void 0 ? void 0 : _e.currency,
        discount: item.discount_amount ? currency.maybeConvertAmount(item.discount_amount, (_f = item.price) === null || _f === void 0 ? void 0 : _f.currency) : 0,
        price: currency.maybeConvertAmount((_g = item === null || item === void 0 ? void 0 : item.price) === null || _g === void 0 ? void 0 : _g.amount, (_h = item.price) === null || _h === void 0 ? void 0 : _h.currency),
        quantity: item.quantity,
        ...(((_j = item === null || item === void 0 ? void 0 : item.variant_options) === null || _j === void 0 ? void 0 : _j.length) ? { item_variant: (item.variant_options || []).join(' / ') } : {}),
      });
    }),
  });
});

/**
 * Handle add to cart event.
 */
window.addEventListener('scAddedToCart', function (e) {
  var _a, _b, _c, _d, _e, _f, _g, _h;
  if (!(window === null || window === void 0 ? void 0 : window.fbq))
    return;
  // get the added item from the event.
  const item = e.detail;
  // sanity check.
  if (!((_a = item === null || item === void 0 ? void 0 : item.price) === null || _a === void 0 ? void 0 : _a.product))
    return;
  const product = (_b = item === null || item === void 0 ? void 0 : item.price) === null || _b === void 0 ? void 0 : _b.product;
  const productCollections = ((_d = (_c = product === null || product === void 0 ? void 0 : product.product_collections) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.map(collection => collection.name)) || [];
  window.fbq('track', 'AddToCart', {
    ...(productCollections.length ? { content_category: productCollections.join(', ') } : {}),
    content_ids: [product.id],
    content_name: (product === null || product === void 0 ? void 0 : product.name) + (((_e = item === null || item === void 0 ? void 0 : item.variant_options) === null || _e === void 0 ? void 0 : _e.length) ? ` - ${item === null || item === void 0 ? void 0 : item.variant_options.join(' / ')}` : ''),
    content_type: 'product',
    contents: [
      {
        id: product.id,
        quantity: item.quantity,
      },
    ],
    currency: (_f = item === null || item === void 0 ? void 0 : item.price) === null || _f === void 0 ? void 0 : _f.currency,
    value: currency.maybeConvertAmount(((_g = item === null || item === void 0 ? void 0 : item.price) === null || _g === void 0 ? void 0 : _g.amount) || 0, ((_h = item === null || item === void 0 ? void 0 : item.price) === null || _h === void 0 ? void 0 : _h.currency) || 'USD'),
  });
});
/**
 * Handle purchase initiated event.
 */
window.addEventListener('scCheckoutInitiated', function (e) {
  var _a, _b, _c, _d;
  if (!(window === null || window === void 0 ? void 0 : window.fbq))
    return;
  const checkout = e.detail;
  window.fbq('track', 'InitiateCheckout', {
    content_ids: (_a = ((checkout === null || checkout === void 0 ? void 0 : checkout.line_items.data) || [])) === null || _a === void 0 ? void 0 : _a.map(item => item.id),
    contents: (_b = ((checkout === null || checkout === void 0 ? void 0 : checkout.line_items.data) || [])) === null || _b === void 0 ? void 0 : _b.map(item => ({ id: item.id, quantity: item.quantity })),
    currency: checkout === null || checkout === void 0 ? void 0 : checkout.currency,
    num_items: ((_d = (_c = checkout === null || checkout === void 0 ? void 0 : checkout.line_items) === null || _c === void 0 ? void 0 : _c.data) === null || _d === void 0 ? void 0 : _d.length) || 0,
    value: currency.maybeConvertAmount(checkout === null || checkout === void 0 ? void 0 : checkout.total_amount, (checkout === null || checkout === void 0 ? void 0 : checkout.currency) || 'USD'),
  });
});
/**
 * Handle purchase complete event.
 */
window.addEventListener('scCheckoutCompleted', function (e) {
  var _a, _b, _c;
  if (!(window === null || window === void 0 ? void 0 : window.fbq))
    return;
  const checkout = e.detail;
  window.fbq('track', 'Purchase', {
    content_ids: (_a = checkout === null || checkout === void 0 ? void 0 : checkout.items) === null || _a === void 0 ? void 0 : _a.map(item => item.item_id),
    content_name: 'Purchase',
    content_type: 'product',
    contents: (_b = checkout === null || checkout === void 0 ? void 0 : checkout.items) === null || _b === void 0 ? void 0 : _b.map(item => ({ id: item.item_id, quantity: item.quantity })),
    currency: checkout === null || checkout === void 0 ? void 0 : checkout.currency,
    num_items: (_c = checkout === null || checkout === void 0 ? void 0 : checkout.items) === null || _c === void 0 ? void 0 : _c.length,
    value: currency.maybeConvertAmount(checkout === null || checkout === void 0 ? void 0 : checkout.total_amount, (checkout === null || checkout === void 0 ? void 0 : checkout.currency) || 'USD'),
  });
});
/**
 * Handle trial started event.
 */
window.addEventListener('scTrialStarted', function (e) {
  if (!(window === null || window === void 0 ? void 0 : window.fbq))
    return;
  const items = e.detail;
  items.forEach(item => {
    var _a, _b, _c;
    window.fbq('track', 'StartTrial', {
      currency: (_a = item.price) === null || _a === void 0 ? void 0 : _a.currency,
      value: currency.maybeConvertAmount(((_b = item.price) === null || _b === void 0 ? void 0 : _b.amount) || 0, ((_c = item.price) === null || _c === void 0 ? void 0 : _c.currency) || 'USD'),
    });
  });
});
/**
 * Handle subscription started event.
 */
window.addEventListener('scSubscriptionStarted', function (e) {
  if (!(window === null || window === void 0 ? void 0 : window.fbq))
    return;
  const items = e.detail;
  items.forEach(item => {
    var _a, _b, _c;
    window.fbq('track', 'Subscribe', {
      currency: (_a = item.price) === null || _a === void 0 ? void 0 : _a.currency,
      value: currency.maybeConvertAmount(((_b = item.price) === null || _b === void 0 ? void 0 : _b.amount) || 0, ((_c = item.price) === null || _c === void 0 ? void 0 : _c.currency) || 'USD'),
    });
  });
});
/**
 * Handle payment info added event.
 */
window.addEventListener('scPaymentInfoAdded', function (e) {
  if (!(window === null || window === void 0 ? void 0 : window.fbq))
    return;
  const detail = e.detail;
  window.fbq('track', 'AddPaymentInfo', {
    content_category: 'Payment Info Added',
    currency: detail === null || detail === void 0 ? void 0 : detail.currency,
  });
});

/** Get the checkout. */
const getCheckout = (formId, mode) => { var _a; return ((_a = store$1.state[mode]) === null || _a === void 0 ? void 0 : _a[formId]) || {}; };
/** Set the checkout. */
const setCheckout = (data, formId) => {
  const mode = (data === null || data === void 0 ? void 0 : data.live_mode) ? 'live' : 'test';
  store$1.set(mode, { ...store$1.state[mode], [formId]: data });
  // update the current checkout state.
  if (state$1.formId === formId && state$1.mode === mode) {
    state$1.checkout = data;
  }
  // set in url only if we are not persisting the cart.
  if (state$1.persist === 'url' && (data === null || data === void 0 ? void 0 : data.id)) {
    window.history.replaceState({}, document.title, addQueryArgs.addQueryArgs(window.location.href, { checkout_id: data === null || data === void 0 ? void 0 : data.id }));
  }
};
/** Clear the order from the store. */
const clearCheckout = (formId, mode) => {
  const { [formId]: remove, ...checkouts } = store$1.state[mode];
  window.history.replaceState({}, document.title, removeQueryArgs(window.location.href, 'redirect_status', 'coupon', 'line_items', 'confirm_checkout_id', 'checkout_id'));
  return store$1.set(mode, checkouts);
};

exports.clearCheckout = clearCheckout;
exports.getCheckout = getCheckout;
exports.on = on$1;
exports.onChange = onChange;
exports.onChange$1 = onChange$1;
exports.removeQueryArgs = removeQueryArgs;
exports.setCheckout = setCheckout;
exports.state = state$1;
exports.store = store;
exports.toggleCart = toggleCart;

//# sourceMappingURL=mutations-65ccc5d3.js.map