'use strict';

const defaultI18n = require('./default-i18n-2d921004.js');
const currency = require('./currency-ba038e2f.js');

const convertAmount = (amount, currency$1) => {
  return currency.zeroDecimalCurrencies.includes(currency$1) ? amount : amount / 100;
};
const getHumanDiscount = (coupon) => {
  if ((coupon === null || coupon === void 0 ? void 0 : coupon.amount_off) && (coupon === null || coupon === void 0 ? void 0 : coupon.currency)) {
    return getFormattedPrice({ amount: coupon.amount_off, currency: coupon.currency });
  }
  if (coupon === null || coupon === void 0 ? void 0 : coupon.percent_off) {
    // Translators: Percent off.
    return defaultI18n.sprintf(defaultI18n.__('%1d%% off', 'surecart'), coupon.percent_off | 0);
  }
  return '';
};
const getFormattedPrice = ({ amount, currency }) => {
  const converted = convertAmount(amount, currency);
  return `${new Intl.NumberFormat(undefined, {
    style: 'currency',
    currency: currency,
  }).format(parseFloat(converted.toFixed(2)))}`;
};
// get the currency symbol for a currency code.
const getCurrencySymbol = (code = 'usd') => {
  var _a;
  const formattedParts = new Intl.NumberFormat(undefined, {
    style: 'currency',
    currency: code,
  }).formatToParts();
  return (_a = formattedParts.find(part => part.type === 'currency')) === null || _a === void 0 ? void 0 : _a.value;
};
const translateInterval = (amount, interval, prefix = defaultI18n.__('every', 'surecart'), fallback = defaultI18n.__('once', 'surecart'), showSingle = false) => {
  switch (interval) {
    case 'day':
      return `${prefix} ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d day', '%d days', amount, 'surecart') : defaultI18n._n('day', '%d days', amount, 'surecart'), amount)}`;
    case 'week':
      return `${prefix} ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d week', '%d weeks', amount, 'surecart') : defaultI18n._n('week', '%d weeks', amount, 'surecart'), amount)}`;
    case 'month':
      return `${prefix} ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d month', '%d months', amount, 'surecart') : defaultI18n._n('month', '%d months', amount, 'surecart'), amount)}`;
    case 'year':
      return `${prefix} ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d year', '%d years', amount, 'surecart') : defaultI18n._n('year', '%d years', amount, 'surecart'), amount)}`;
    default:
      return fallback;
  }
};
const translateAbbreviatedInterval = (amount, interval, fallback = defaultI18n.__('once', 'surecart'), showSingle = false) => {
  switch (interval) {
    case 'day':
      return ` / ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d day', '%d days', amount, 'surecart') : defaultI18n._n('day', '%d days', amount, 'surecart'), amount)}`;
    case 'week':
      return ` / ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d wk', '%d wks', amount, 'surecart') : defaultI18n._n('wk', '%d wks', amount, 'surecart'), amount)}`;
    case 'month':
      return ` / ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d mo', '%d months', amount, 'surecart') : defaultI18n._n('mo', '%d mos', amount, 'surecart'), amount)}`;
    case 'year':
      return ` / ${defaultI18n.sprintf(showSingle ? defaultI18n._n('%d yr', '%d yrs', amount, 'surecart') : defaultI18n._n('yr', '%d yrs', amount, 'surecart'), amount)}`;
    default:
      return fallback;
  }
};
const intervalString = (price, options = {}) => {
  if (!price) {
    return '';
  }
  const { showOnce, labels, abbreviate } = options;
  const { interval = defaultI18n.__('every', 'surecart') } = labels || {};
  return `${intervalCountString(price, interval, !!showOnce ? defaultI18n.__('once', 'surecart') : '', abbreviate)} ${periodCountString(price, abbreviate)}`;
};
const intervalCountString = (price, prefix, fallback = defaultI18n.__('once', 'surecart'), abbreviate = false) => {
  if (!price.recurring_interval_count || !price.recurring_interval || 1 === (price === null || price === void 0 ? void 0 : price.recurring_period_count)) {
    return '';
  }
  if (abbreviate) {
    return translateAbbreviatedInterval(price.recurring_interval_count, price.recurring_interval, fallback);
  }
  return translateInterval(price.recurring_interval_count, price.recurring_interval, ` ${prefix}`, fallback);
};
const periodCountString = (price, abbreviate = false) => {
  if (!(price === null || price === void 0 ? void 0 : price.recurring_period_count) || 1 === (price === null || price === void 0 ? void 0 : price.recurring_period_count)) {
    return '';
  }
  if (abbreviate) {
    return `x ${price.recurring_period_count}`;
  }
  return ` (${defaultI18n.sprintf(defaultI18n._n('%d payment', '%d payments', price.recurring_period_count, 'surecart'), price.recurring_period_count)})`;
};
const translateRemainingPayments = payments => {
  return defaultI18n.sprintf(defaultI18n._n('%d payment remaining', '%d payments remaining', payments, 'surecart'), payments);
};
const productNameWithPrice = price => {
  var _a;
  if (!price) {
    return '';
  }
  return `${(_a = price === null || price === void 0 ? void 0 : price.product) === null || _a === void 0 ? void 0 : _a.name} ${(price === null || price === void 0 ? void 0 : price.name) ? `— ${price.name}` : ''}`;
};
const getHumanDiscountRedeemableStatus = status => {
  switch (status) {
    case 'invalid':
      return defaultI18n.__('Not valid', 'surecart');
    case 'expired':
      return defaultI18n.__('Expired', 'surecart');
    case 'gone':
      return defaultI18n.__('Not available', 'surecart');
    case 'less_than_min_subtotal_amount':
      return defaultI18n.__('Minimum not met', 'surecart');
    case 'not_applicable':
      return defaultI18n.__('Product(s) not eligible', 'surecart');
    case 'not_applicable_to_customer':
      return defaultI18n.__('Not eligible', 'surecart');
    case '':
      return '';
    default:
      return defaultI18n.__('Not redeemable', 'surecart');
  }
};

exports.getCurrencySymbol = getCurrencySymbol;
exports.getFormattedPrice = getFormattedPrice;
exports.getHumanDiscount = getHumanDiscount;
exports.getHumanDiscountRedeemableStatus = getHumanDiscountRedeemableStatus;
exports.intervalString = intervalString;
exports.productNameWithPrice = productNameWithPrice;
exports.translateInterval = translateInterval;
exports.translateRemainingPayments = translateRemainingPayments;

//# sourceMappingURL=price-a28ce962.js.map