/**
 * External dependencies.
 */
import { h } from '@stencil/core';
import { addQueryArgs, getQueryArgs } from '@wordpress/url';
import { __, sprintf } from '@wordpress/i18n';
import { speak } from '@wordpress/a11y';
import apiFetch, { handleNonceError } from '../../../../functions/fetch';
import '@store/product/facebook';
import '@store/product/google';
export class ScProductItemList {
  constructor() {
    this.ids = undefined;
    this.sort = 'created_at:desc';
    this.query = undefined;
    this.searchEnabled = true;
    this.sortEnabled = true;
    this.collectionEnabled = true;
    this.collectionId = undefined;
    this.pageTitle = undefined;
    this.featured = false;
    this.paginationEnabled = true;
    this.ajaxPagination = true;
    this.paginationAutoScroll = true;
    this.layoutConfig = undefined;
    this.paginationAlignment = 'center';
    this.limit = 15;
    this.page = 1;
    this.products = undefined;
    this.loading = false;
    this.busy = false;
    this.error = undefined;
    this.currentPage = 1;
    this.currentQuery = undefined;
    this.pagination = {
      total: 0,
      total_pages: 0,
    };
    this.collections = undefined;
    this.selectedCollections = [];
  }
  handleProductsChanged(newProducts, oldProducts) {
    var _a;
    const productIds = new Set([...(oldProducts || []).map(product => product.id), ...(newProducts || []).map(product => product.id)]);
    if ((newProducts === null || newProducts === void 0 ? void 0 : newProducts.length) === (oldProducts === null || oldProducts === void 0 ? void 0 : oldProducts.length) && productIds.size === newProducts.length) {
      return;
    }
    const title = [
      this.pageTitle,
      this.paginationEnabled ? sprintf(__('Page %d', 'surecart'), this.currentPage) : undefined,
      this.sort ? this.renderSortName() : undefined,
      this.query || ((_a = this.selectedCollections) === null || _a === void 0 ? void 0 : _a.length) ? __('Search results', 'surecart') : undefined,
    ]
      .filter(item => !!item)
      .join(' - ');
    this.scProductsViewed.emit({
      products: this.products,
      pageTitle: title,
      collectionId: this.collectionId,
    });
  }
  componentWillLoad() {
    var _a;
    if (!((_a = this === null || this === void 0 ? void 0 : this.products) === null || _a === void 0 ? void 0 : _a.length)) {
      this.getProducts();
    }
    else {
      this.handleProductsChanged(this.products);
    }
    if (this.collectionEnabled) {
      this.getCollections();
    }
  }
  // Append URL if no 'product-page' found
  doPagination(page) {
    // handle ajax pagination
    if (this.ajaxPagination) {
      this.page = page;
      this.updateProducts();
      this.paginationAutoScroll && this.el.scrollIntoView({ behavior: 'smooth' });
      return;
    }
    // handle server pagination.
    const newUrl = addQueryArgs(location.href, { 'product-page': page });
    window.location.replace(newUrl);
  }
  // Fetch all products
  async getProducts() {
    const { 'product-page': page } = getQueryArgs(window.location.href);
    this.page = this.paginationEnabled && page ? parseInt(page) : 1;
    try {
      this.loading = true;
      await this.fetchProducts();
    }
    catch (error) {
      console.error(error);
    }
    finally {
      this.loading = false;
    }
  }
  // Fetch all collections
  async getCollections() {
    try {
      this.collections = (await apiFetch({
        path: addQueryArgs('surecart/v1/product_collections/', {
          per_page: 100,
        }),
      }));
    }
    catch (error) {
      console.error(error);
    }
  }
  async handleSortChange() {
    this.page = 1;
    this.updateProducts();
  }
  async updateProducts(emitSearchEvent = false) {
    var _a;
    try {
      this.busy = true;
      await this.fetchProducts();
      if (!!this.query && emitSearchEvent) {
        this.scSearched.emit({
          searchString: this.query,
          searchResultCount: (_a = this.products) === null || _a === void 0 ? void 0 : _a.length,
          searchResultIds: this.products.map(product => product.id),
        });
      }
    }
    catch (error) {
      console.log('error');
      console.error(error);
      this.error = error.message || __('An unknown error occurred.', 'surecart');
    }
    finally {
      this.busy = false;
    }
  }
  handleIdsChange() {
    if (this.debounce !== null) {
      clearTimeout(this.debounce);
      this.debounce = null;
    }
    this.debounce = window.setTimeout(() => {
      // your debounced traitment
      this.updateProducts();
      this.debounce = null;
    }, 200);
  }
  async fetchProducts() {
    var _a, _b;
    let collectionIds = ((_a = this.selectedCollections) === null || _a === void 0 ? void 0 : _a.map(collection => collection.id)) || [];
    // If we have a collectionId, we should only fetch products from that collection.
    if (this.collectionId) {
      collectionIds = [this.collectionId];
    }
    try {
      const response = (await apiFetch({
        path: addQueryArgs(`surecart/v1/products/`, {
          expand: ['prices', 'featured_product_media', 'product_medias', 'product_media.media', 'variants'],
          archived: false,
          status: ['published'],
          per_page: this.limit,
          page: this.page,
          sort: this.sort,
          product_collection_ids: collectionIds,
          ...(this.featured ? { featured: true } : {}),
          ...(((_b = this.ids) === null || _b === void 0 ? void 0 : _b.length) ? { ids: this.ids } : {}),
          ...(this.query ? { query: this.query } : {}),
        }),
        parse: false,
      }));
      this.currentQuery = this.query;
      this.pagination = {
        total: parseInt(response.headers.get('X-WP-Total')),
        total_pages: parseInt(response.headers.get('X-WP-TotalPages')),
      };
      this.products = (await response.json());
      if (!!collectionIds.length || !!this.query) {
        speak(sprintf(__('%s products found', 'surecart'), this.pagination.total));
      }
    }
    catch (response) {
      // we will want to handle nonce error if we are bypassing the apiFetch parser.
      await handleNonceError(response)
        .then(() => this.fetchProducts())
        .catch(error => {
        this.error = error.message || __('An unknown error occurred.', 'surecart');
      });
    }
  }
  renderSortName() {
    switch (this.sort) {
      case 'created_at:desc':
        return __('Latest', 'surecart');
      case 'created_at:asc':
        return __('Oldest', 'surecart');
      case 'name:asc':
        return __('Alphabetical, A-Z', 'surecart');
      case 'name:desc':
        return __('Alphabetical, Z-A', 'surecart');
      default:
        return __('Sort', 'surecart');
    }
  }
  toggleSelectCollection(collection) {
    // if collection not in selectedCollections, add it, otherwise remove it
    if (!this.selectedCollections.find(c => c.id === collection.id)) {
      this.selectedCollections = [...this.selectedCollections, collection];
    }
    else {
      this.selectedCollections = this.selectedCollections.filter(c => c.id !== collection.id);
    }
  }
  getCollectionsAfterFiltered() {
    var _a;
    return ((_a = this.collections) !== null && _a !== void 0 ? _a : []).filter(collection => {
      return !this.selectedCollections.some(selected => selected.id === collection.id);
    });
  }
  isPaginationAvailable() {
    var _a;
    return !!((_a = this.products) === null || _a === void 0 ? void 0 : _a.length) && this.pagination.total > this.products.length && this.paginationEnabled;
  }
  render() {
    var _a, _b, _c, _d, _e, _f;
    return (h("div", { class: {
        'product-item-list__wrapper': true,
        'product-item-list__has-search': !!this.query,
      } }, this.error && (h("sc-alert", { type: "danger", open: true }, this.error)), (this.searchEnabled || this.sortEnabled || this.collectionEnabled) && (h("div", { class: "product-item-list__header" }, h("div", { class: "product-item-list__controls" }, h("div", { class: "product-item-list__sort" }, this.sortEnabled && (h("sc-dropdown", { style: { '--panel-width': '15em' } }, h("sc-button", { type: "text", caret: true, slot: "trigger" }, h("sc-visually-hidden", null, __('Dropdown to sort products.', 'surecart'), " "), this.renderSortName(), h("sc-visually-hidden", null, " ", __('selected.', 'surecart'))), h("sc-menu", { "aria-label": __('Sort Products', 'surecart') }, h("sc-menu-item", { "aria-label": __('Sort by latest', 'surecart'), onClick: () => (this.sort = 'created_at:desc') }, __('Latest', 'surecart')), h("sc-menu-item", { "aria-label": __('Sort by oldest', 'surecart'), onClick: () => (this.sort = 'created_at:asc') }, __('Oldest', 'surecart')), h("sc-menu-item", { "aria-label": __('Sort by name, A to Z', 'surecart'), onClick: () => (this.sort = 'name:asc') }, __('Alphabetical, A-Z', 'surecart')), h("sc-menu-item", { "aria-label": __('Sort by name, Z to A', 'surecart'), onClick: () => (this.sort = 'name:desc') }, __('Alphabetical, Z-A', 'surecart'))))), this.collectionEnabled && ((_a = this.collections) !== null && _a !== void 0 ? _a : []).length > 0 && (h("sc-dropdown", { style: { '--panel-width': '15rem' } }, h("sc-button", { type: "text", caret: true, slot: "trigger" }, h("sc-visually-hidden", null, sprintf(__('Dropdown to filter products by collection. %s selected.', 'surecart'), ((_b = this.selectedCollections) === null || _b === void 0 ? void 0 : _b.length) ? this.selectedCollections.map(collection => collection === null || collection === void 0 ? void 0 : collection.name).join(',') : __('None', 'surecart'))), h("span", { "aria-hidden": true }, " ", __('Filter', 'surecart'))), h("sc-menu", { "aria-label": __('Filter products', 'surecart') }, ((_c = this.collections) !== null && _c !== void 0 ? _c : []).map(collection => {
      return (h("sc-menu-item", { checked: this.selectedCollections.some(selected => (selected === null || selected === void 0 ? void 0 : selected.id) === (collection === null || collection === void 0 ? void 0 : collection.id)), onClick: () => this.toggleSelectCollection(collection), key: collection === null || collection === void 0 ? void 0 : collection.id, "aria-label": sprintf(__('Filter by %s', 'surecart'), collection === null || collection === void 0 ? void 0 : collection.name) }, collection.name));
    }))))), h("div", { class: "product-item-list__search" }, this.searchEnabled &&
      (((_d = this.query) === null || _d === void 0 ? void 0 : _d.length) && this.query === this.currentQuery ? (h("div", { class: "product-item-list__search-tag" }, h("div", { class: "product-item-list__search-label" }, __('Search Results:', 'surecart')), h("sc-tag", { clearable: true, onScClear: () => {
          this.query = '';
          this.currentQuery = '';
          this.updateProducts();
        }, "aria-label": sprintf(__('Searched for %s. Press space to clear search.', 'surecart'), this.query) }, this.query))) : (h("sc-input", { type: "text", placeholder: __('Search', 'surecart'), size: "small", onKeyUp: e => {
          if (e.key === 'Enter') {
            this.query = e.target.value;
            this.updateProducts(true);
          }
        }, value: this.query, clearable: true }, this.query ? (h("sc-icon", { class: "clear-button", slot: "prefix", name: "x", onClick: () => {
          this.query = '';
        } })) : (h("sc-icon", { slot: "prefix", name: "search" })), h("sc-button", { class: "search-button", type: "link", slot: "suffix", busy: this.busy, onClick: () => {
          this.updateProducts(true);
        } }, __('Search', 'surecart'))))))), this.collectionEnabled && this.selectedCollections.length > 0 && (h("div", { class: "product-item-list__search-tag" }, this.selectedCollections.map(collection => (h("sc-tag", { key: collection === null || collection === void 0 ? void 0 : collection.id, clearable: true, onScClear: () => {
        this.toggleSelectCollection(collection);
      } }, collection === null || collection === void 0 ? void 0 : collection.name))))))), !((_e = this.products) === null || _e === void 0 ? void 0 : _e.length) && !this.loading && (h("sc-empty", { class: "product-item-list__empty", icon: "shopping-bag" }, __('No products found.', 'surecart'))), h("section", { class: "product-item-list", "aria-label": __('Product list', 'surecart') }, this.loading
      ? [...Array(((_f = this.products) === null || _f === void 0 ? void 0 : _f.length) || this.limit || 10)].map((_, index) => (h("div", { class: "product-item-list__loader", key: index }, (this.layoutConfig || []).map(layout => {
        var _a, _b;
        switch (layout.blockName) {
          case 'surecart/product-item-title':
            return (h("div", { style: {
                textAlign: 'var(--sc-product-title-align)',
              } }, h("sc-skeleton", { style: {
                width: '80%',
                display: 'inline-block',
              } })));
          case 'surecart/product-item-image':
            return (h("sc-skeleton", { style: {
                'width': '100%',
                'minHeight': '90%',
                'aspectRatio': (_b = (_a = layout.attributes) === null || _a === void 0 ? void 0 : _a.ratio) !== null && _b !== void 0 ? _b : '1/1.4',
                '--sc-border-radius-pill': '12px',
                'display': 'inline-block',
              } }));
          case 'surecart/product-item-price':
            return (h("div", { style: {
                textAlign: 'var(--sc-product-price-align)',
              } }, h("sc-skeleton", { style: {
                width: '40%',
                display: 'inline-block',
              } })));
          default:
            return null;
        }
      }))))
      : (this.products || []).map((product, index) => {
        return (h("sc-product-item", { key: product === null || product === void 0 ? void 0 : product.id, ...(this.products.length - 1 === index
            ? {
              'aria-label': sprintf(__('You have reached the end of product list. %s', 'surecart'), this.isPaginationAvailable() ? __('Press tab to browse more products using pagination.', 'surecart') : __('No more products to browse.', 'surecart')),
            }
            : {}), exportparts: "title, price, image", product: product, layoutConfig: this.layoutConfig }));
      })), this.isPaginationAvailable() && (h("div", { class: {
        'product-item-list__pagination': true,
        '--is-aligned-left': this.paginationAlignment === 'left',
        '--is-aligned-center': this.paginationAlignment === 'center',
        '--is-aligned-right': this.paginationAlignment === 'right',
      } }, h("sc-pagination", { page: this.page, perPage: this.limit, total: this.pagination.total, totalPages: this.pagination.total_pages, totalShowing: this.limit, onScNextPage: () => this.doPagination(this.page + 1), onScPrevPage: () => this.doPagination(this.page - 1) }))), (this.busy || this.loading) && h("sc-block-ui", null)));
  }
  static get is() { return "sc-product-item-list"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["sc-product-item-list.scss"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["sc-product-item-list.css"]
    };
  }
  static get properties() {
    return {
      "ids": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "string[]",
          "resolved": "string[]",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Limit to a set of ids."
        }
      },
      "sort": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Sort"
        },
        "attribute": "sort",
        "reflect": false,
        "defaultValue": "'created_at:desc'"
      },
      "query": {
        "type": "string",
        "mutable": true,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Query to search for"
        },
        "attribute": "query",
        "reflect": false
      },
      "searchEnabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Should allow search"
        },
        "attribute": "search-enabled",
        "reflect": false,
        "defaultValue": "true"
      },
      "sortEnabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Should allow search"
        },
        "attribute": "sort-enabled",
        "reflect": false,
        "defaultValue": "true"
      },
      "collectionEnabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Should allow collection filter"
        },
        "attribute": "collection-enabled",
        "reflect": false,
        "defaultValue": "true"
      },
      "collectionId": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Show for a specific collection"
        },
        "attribute": "collection-id",
        "reflect": false
      },
      "pageTitle": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "The page title"
        },
        "attribute": "page-title",
        "reflect": false
      },
      "featured": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Show only featured products."
        },
        "attribute": "featured",
        "reflect": false,
        "defaultValue": "false"
      },
      "paginationEnabled": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Should we paginate?"
        },
        "attribute": "pagination-enabled",
        "reflect": false,
        "defaultValue": "true"
      },
      "ajaxPagination": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Should we paginate?"
        },
        "attribute": "ajax-pagination",
        "reflect": false,
        "defaultValue": "true"
      },
      "paginationAutoScroll": {
        "type": "boolean",
        "mutable": false,
        "complexType": {
          "original": "boolean",
          "resolved": "boolean",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Should we auto-scroll to the top when paginating via ajax"
        },
        "attribute": "pagination-auto-scroll",
        "reflect": false,
        "defaultValue": "true"
      },
      "layoutConfig": {
        "type": "unknown",
        "mutable": false,
        "complexType": {
          "original": "LayoutConfig",
          "resolved": "{ blockName: string; attributes: any; }[]",
          "references": {
            "LayoutConfig": {
              "location": "local",
              "path": "/Users/andre/sites/surecart/wp-content/plugins/surecart/packages/components/src/components/controllers/products/sc-product-item-list/sc-product-item-list.tsx"
            }
          }
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "paginationAlignment": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "pagination-alignment",
        "reflect": false,
        "defaultValue": "'center'"
      },
      "limit": {
        "type": "number",
        "mutable": false,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "limit",
        "reflect": false,
        "defaultValue": "15"
      },
      "page": {
        "type": "number",
        "mutable": true,
        "complexType": {
          "original": "number",
          "resolved": "number",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": ""
        },
        "attribute": "page",
        "reflect": false,
        "defaultValue": "1"
      },
      "products": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "Product[]",
          "resolved": "Product[]",
          "references": {
            "Product": {
              "location": "import",
              "path": "../../../../types"
            }
          }
        },
        "required": false,
        "optional": true,
        "docs": {
          "tags": [],
          "text": ""
        }
      },
      "pagination": {
        "type": "unknown",
        "mutable": true,
        "complexType": {
          "original": "{\n    total: number;\n    total_pages: number;\n  }",
          "resolved": "{ total: number; total_pages: number; }",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Pagination"
        },
        "defaultValue": "{\n    total: 0,\n    total_pages: 0,\n  }"
      }
    };
  }
  static get states() {
    return {
      "loading": {},
      "busy": {},
      "error": {},
      "currentPage": {},
      "currentQuery": {},
      "collections": {},
      "selectedCollections": {}
    };
  }
  static get events() {
    return [{
        "method": "scSearched",
        "name": "scSearched",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Product was searched"
        },
        "complexType": {
          "original": "ProductsSearchedParams",
          "resolved": "ProductsSearchedParams",
          "references": {
            "ProductsSearchedParams": {
              "location": "import",
              "path": "../../../../types"
            }
          }
        }
      }, {
        "method": "scProductsViewed",
        "name": "scProductsViewed",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [],
          "text": "Products viewed"
        },
        "complexType": {
          "original": "ProductsViewedParams",
          "resolved": "ProductsViewedParams",
          "references": {
            "ProductsViewedParams": {
              "location": "import",
              "path": "../../../../types"
            }
          }
        }
      }];
  }
  static get elementRef() { return "el"; }
  static get watchers() {
    return [{
        "propName": "products",
        "methodName": "handleProductsChanged"
      }, {
        "propName": "sort",
        "methodName": "handleSortChange"
      }, {
        "propName": "selectedCollections",
        "methodName": "handleSortChange"
      }, {
        "propName": "query",
        "methodName": "handleSortChange"
      }, {
        "propName": "ids",
        "methodName": "handleIdsChange"
      }, {
        "propName": "limit",
        "methodName": "handleIdsChange"
      }, {
        "propName": "featured",
        "methodName": "handleIdsChange"
      }];
  }
}
//# sourceMappingURL=sc-product-item-list.js.map
