/**
 * Internal dependencies.
 */
import state from './store';
import { getSerializedState, isProductVariantOptionMissing, isProductVariantOptionSoldOut } from '@store/utils';
import { getVariantFromValues } from '../../functions/util';
/**
 * Available product prices
 *
 * @param {string} productId - Product ID
 *
 * @returns {Price[]} - Returns an array of prices that are not archived
 */
export const availablePrices = (productId) => { var _a; return (((_a = state[productId]) === null || _a === void 0 ? void 0 : _a.prices) || []).filter(price => !(price === null || price === void 0 ? void 0 : price.archived)).sort((a, b) => (a === null || a === void 0 ? void 0 : a.position) - (b === null || b === void 0 ? void 0 : b.position)); }; // sort by position
/**
 * Get Product
 */
export const getProduct = (productId) => { var _a; return (_a = state[productId]) !== null && _a !== void 0 ? _a : null; };
/**
 * Check if product label stock is enabled and not out of stock purchases are allowed.
 *
 * @returns {boolean} - Returns stock needs to be checked or not
 */
export const isStockNeedsToBeChecked = (productId) => { var _a, _b, _c, _d; return !!(((_b = (_a = state === null || state === void 0 ? void 0 : state[productId]) === null || _a === void 0 ? void 0 : _a.product) === null || _b === void 0 ? void 0 : _b.stock_enabled) && !((_d = (_c = state === null || state === void 0 ? void 0 : state[productId]) === null || _c === void 0 ? void 0 : _c.product) === null || _d === void 0 ? void 0 : _d.allow_out_of_stock_purchases)); };
/**
 * Check if this option is out of stock base on the selected variant.
 */
export const isOptionSoldOut = (productId, optionNumber, option) => isProductVariantOptionSoldOut(optionNumber, option, state[productId].variantValues, state[productId].product);
/**
 * Check if this option is out of stock base on the selected variant.
 */
export const isOptionMissing = (productId, optionNumber, option) => isProductVariantOptionMissing(optionNumber, option, state[productId].variantValues, state[productId].product);
/**
 * Is product out of stock.
 *
 * @returns {boolean} - Returns true if product is out of stock
 */
export const isProductOutOfStock = (productId) => {
  var _a, _b, _c, _d;
  // If stock doesn't need to be checked, product is not out of stock.
  if (!isStockNeedsToBeChecked(productId)) {
    return false;
  }
  // If no variant is selected, check against product stock.
  if (!((_b = (_a = state === null || state === void 0 ? void 0 : state[productId]) === null || _a === void 0 ? void 0 : _a.selectedVariant) === null || _b === void 0 ? void 0 : _b.id))
    return ((_c = state[productId].product) === null || _c === void 0 ? void 0 : _c.available_stock) <= 0;
  // Check against selected variant's stock.
  return ((_d = state[productId].selectedVariant) === null || _d === void 0 ? void 0 : _d.available_stock) <= 0;
};
/**
 * Is the selected variant missing.
 */
export const isSelectedVariantMissing = (productId) => { var _a, _b, _c; return !!((_b = (_a = state === null || state === void 0 ? void 0 : state[productId]) === null || _a === void 0 ? void 0 : _a.variants) === null || _b === void 0 ? void 0 : _b.length) && ((_c = getVariantFromValues({ variants: state[productId].variants, values: state[productId].variantValues })) === null || _c === void 0 ? void 0 : _c.id) === undefined; };
/**
 * Get product default state
 *
 * @returns {ProductState} - Returns the product state
 */
export const getDefaultState = () => {
  const { product: serializedProductState = {} } = getSerializedState();
  return (Object.values(serializedProductState).reduce((acc, productState) => {
    const { selectedPrice, product, selectedVariant } = productState || {};
    const current = {
      ...productState,
      quantity: 1,
      total: null,
      dialog: null,
      busy: false,
      error: null,
      adHocAmount: (selectedPrice === null || selectedPrice === void 0 ? void 0 : selectedPrice.amount) || null,
      disabled: (selectedPrice === null || selectedPrice === void 0 ? void 0 : selectedPrice.archived) || (product === null || product === void 0 ? void 0 : product.archived),
      line_item: {
        price_id: selectedPrice === null || selectedPrice === void 0 ? void 0 : selectedPrice.id,
        quantity: 1,
        ...((selectedPrice === null || selectedPrice === void 0 ? void 0 : selectedPrice.ad_hoc) ? { ad_hoc_amount: selectedPrice === null || selectedPrice === void 0 ? void 0 : selectedPrice.amount } : {}),
        ...((selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.id) ? { variant_id: selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.id } : {}),
      },
      variantValues: {
        ...((selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.option_1) ? { option_1: selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.option_1 } : {}),
        ...((selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.option_2) ? { option_2: selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.option_2 } : {}),
        ...((selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.option_3) ? { option_3: selectedVariant === null || selectedVariant === void 0 ? void 0 : selectedVariant.option_3 } : {}),
      },
    };
    acc[product.id] = current;
    return acc;
  }, {}) || {});
};
export const availableSubscriptionPrices = (productId) => (availablePrices(productId) || []).filter(price => price === null || price === void 0 ? void 0 : price.recurring_interval).sort((a, b) => (a === null || a === void 0 ? void 0 : a.position) - (b === null || b === void 0 ? void 0 : b.position));
export const availableNonSubscriptionPrices = (productId) => (availablePrices(productId) || []).filter(price => !(price === null || price === void 0 ? void 0 : price.recurring_interval)).sort((a, b) => (a === null || a === void 0 ? void 0 : a.position) - (b === null || b === void 0 ? void 0 : b.position));
//# sourceMappingURL=getters.js.map
